@@ -24,17 +24,30 @@
 #include <limits.h>
 #include <ctype.h>
 
+#include <gconf/gconf-client.h>
 #include <panel-applet-gconf.h>
 #include <egg-screen-help.h>
 #include "geyes.h"
 
-gchar *theme_directories[] = {
-        GEYES_THEMES_DIR,
-        "~/.gnome/geyes-themes/"
-};
 #define NUM_THEME_DIRECTORIES 2
 #define HIG_IDENTATION  "    "
 
+gchar *theme_directories[NUM_THEME_DIRECTORIES];
+
+void
+theme_dirs_create ()
+{
+	static themes_created = FALSE;
+	
+	if (themes_created == TRUE)
+		return;
+
+	theme_directories[0] = g_strdup (GEYES_THEMES_DIR);
+	theme_directories[1] = g_strdup_printf ("%s/.gnome2/geyes-themes/", g_get_home_dir());
+
+	themes_created = TRUE;
+}
+
 static void
 parse_theme_file (EyesApplet *eyes_applet, FILE *theme_file)
 {
@@ -56,6 +69,8 @@ parse_theme_file (EyesApplet *eyes_applet, FILE *theme_file)
                                 token++;
                         }
                         sscanf (token, "%d", &eyes_applet->num_eyes);
+			if (eyes_applet->num_eyes > MAX_EYES)
+				eyes_applet->num_eyes = MAX_EYES;
                 } else if (strncmp (token, "eye-pixmap", strlen ("eye-pixmap")) == 0) {
                         token = strtok (NULL, "\"");
                         token = strtok (NULL, "\"");          
@@ -78,9 +93,11 @@ parse_theme_file (EyesApplet *eyes_applet, FILE *theme_file)
         }       
 }
 
-void
+int
 load_theme (EyesApplet *eyes_applet, const gchar *theme_dir)
 {
+	GtkWidget *dialog;
+
 	FILE* theme_file;
         gchar *file_name;
 
@@ -95,7 +112,27 @@ load_theme (EyesApplet *eyes_applet, const gchar *theme_dir)
                 file_name = g_strdup (GEYES_THEMES_DIR "Default-tiny/config");
                 theme_file = fopen (file_name, "r");
         }
-        
+
+	/* if it's still NULL we've got a major problem */
+	if (theme_file == NULL) {
+		dialog = gtk_message_dialog_new_with_markup (NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK,
+				"<b>%s</b>\n\n%s",
+				_("Can not launch the eyes applet."),
+				_("There was a fatal error while trying to load the theme."));
+
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		gtk_widget_destroy (GTK_WIDGET (eyes_applet->applet));
+
+		return FALSE;
+	}
+
         parse_theme_file (eyes_applet, theme_file);
         fclose (theme_file);
 
@@ -114,7 +151,8 @@ load_theme (EyesApplet *eyes_applet, const gchar *theme_dir)
         eyes_applet->pupil_width = gdk_pixbuf_get_width (eyes_applet->pupil_image);
         
         g_free (file_name);
-        
+   
+	return TRUE;
 }
 
 static void
@@ -134,6 +172,25 @@ destroy_theme (EyesApplet *eyes_applet)
         g_free (eyes_applet->theme_name);
 }
 
+static gboolean
+key_writable (PanelApplet *applet, const char *key)
+{
+	gboolean writable;
+	char *fullkey;
+	static GConfClient *client = NULL;
+
+	if (client == NULL)
+		client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	writable = gconf_client_key_is_writable (client, fullkey, NULL);
+
+	g_free (fullkey);
+
+	return writable;
+}
+
 static void
 theme_selected_cb (GtkTreeSelection *selection, gpointer data)
 {
@@ -151,7 +208,7 @@ theme_selected_cb (GtkTreeSelection *selection, gpointer data)
 	g_return_if_fail (theme);
 	
 	theme_dir = g_strdup_printf ("%s/", theme);
-	if (!g_strcasecmp (theme_dir, eyes_applet->theme_dir)) {
+	if (!g_ascii_strncasecmp (theme_dir, eyes_applet->theme_dir, strlen (theme_dir))) {
 		g_free (theme_dir);
 		return;
 	}
@@ -245,6 +302,7 @@ properties_cb (BonoboUIComponent *uic,
         gtk_dialog_set_default_response(GTK_DIALOG (pbox), GTK_RESPONSE_CLOSE);
         gtk_dialog_set_has_separator (GTK_DIALOG (pbox), FALSE);
         gtk_container_set_border_width (GTK_CONTAINER (pbox), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (pbox)->vbox), 2);
 
         g_signal_connect (pbox, "response",
 			  G_CALLBACK (presponse_cb),
@@ -302,10 +360,15 @@ properties_cb (BonoboUIComponent *uic,
         gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
                                                            
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-        g_signal_connect (selection, "changed",
-        		  G_CALLBACK (theme_selected_cb),
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (theme_selected_cb),
 			  eyes_applet);
-        
+
+	if ( ! key_writable (eyes_applet->applet, "theme_path")) {
+		gtk_widget_set_sensitive (tree, FALSE);
+		gtk_widget_set_sensitive (label, FALSE);
+	}
+
         for (i = 0; i < NUM_THEME_DIRECTORIES; i++) {
                 if ((dfd = opendir (theme_directories[i])) != NULL) {
                         while ((dp = readdir (dfd)) != NULL) {
@@ -323,7 +386,8 @@ properties_cb (BonoboUIComponent *uic,
                                         gtk_list_store_insert (model, &iter, 0);
                                         gtk_list_store_set (model, &iter, 0, &filename, -1);
                                         theme_dir = g_strdup_printf ("%s/", filename);
-                                        if (!g_strcasecmp (eyes_applet->theme_dir, theme_dir)) {
+                                        
+					if (!g_ascii_strncasecmp (eyes_applet->theme_dir, theme_dir, strlen (theme_dir))) {
                                         	GtkTreePath *path;
                                         	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), 
                                                         			&iter);
