@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.71 2004/01/13 07:11:21 arvind Exp $ */
+/* $Id: gweather-applet.c,v 1.78 2004/04/28 13:29:46 gowen Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -30,46 +30,69 @@
 #include "gweather-dialog.h"
 #include "gweather-applet.h"
 
-/* FIX - This code is WAY too kludgy!... */
 static void place_widgets (GWeatherApplet *gw_applet)
 {
-    gint size = gw_applet->size;
+    PangoLayout *panlay;
+    int size = gw_applet->size;
     const gchar *temp;   
- 
+    int textheight, textwidth;
+    int picheight = 0, picwidth = 0;
+
     if (gw_applet->box)
-        gtk_widget_destroy (gw_applet->box);
-    
-    if (((gw_applet->orient == PANEL_APPLET_ORIENT_LEFT) || 
-         (gw_applet->orient == PANEL_APPLET_ORIENT_RIGHT)) ^ (size < 25)) {
-         gw_applet->box = gtk_hbox_new (FALSE, 2);
-         
-    }
-    else {
-         gw_applet->box = gtk_vbox_new (FALSE, 2);
-    }
-    
-    gtk_container_add (GTK_CONTAINER (gw_applet->applet), gw_applet->box);
+	gtk_widget_destroy (gw_applet->box);
     
+    gw_applet->label = gtk_label_new("0\302\260F");
+
+    /* Create the weather icon */
     weather_info_get_pixbuf_mini(gw_applet->gweather_info, 
     				 &(gw_applet->applet_pixbuf));     
     gw_applet->image = gtk_image_new_from_pixbuf (gw_applet->applet_pixbuf);
-    gtk_box_pack_start (GTK_BOX (gw_applet->box), gw_applet->image, FALSE, FALSE, 0);
-         
-    gw_applet->label = gtk_label_new("0\302\260F");
-    gtk_box_pack_start (GTK_BOX (gw_applet->box), gw_applet->label, FALSE, FALSE, 0);
-    
     gtk_image_set_from_pixbuf (GTK_IMAGE (gw_applet->image), 
     			       gw_applet->applet_pixbuf);
 
-    /* Update temperature text */
+    /* We check against the weather indication icon sizes for box reordering */
+    if (gw_applet->applet_pixbuf != NULL) {
+	picwidth = gdk_pixbuf_get_width(gw_applet->applet_pixbuf);
+	picheight = gdk_pixbuf_get_height(gw_applet->applet_pixbuf);
+    }
+    
+    /* We check against text sizes for box reordering */
+    panlay = gtk_label_get_layout(GTK_LABEL (gw_applet->label));
+    pango_layout_get_pixel_size(panlay, &textwidth, &textheight);
 
-    temp = weather_info_get_temp_summary(gw_applet->gweather_info);
-    if (temp) {
-    	gtk_label_set_text(GTK_LABEL(gw_applet->label), temp);
+    /* Set the right box for the dimension and panel type */
+    if (gw_applet->orient == PANEL_APPLET_ORIENT_UP || gw_applet->orient == PANEL_APPLET_ORIENT_DOWN) {
+	size = gw_applet->container->allocation.height;
+
+	if (size < picheight + textheight + 9)
+		gw_applet->box = gtk_hbox_new (FALSE, 2);
+	else
+		gw_applet->box = gtk_vbox_new (FALSE, 2);
+	
+    } else if (gw_applet->orient == PANEL_APPLET_ORIENT_RIGHT || gw_applet->orient == PANEL_APPLET_ORIENT_LEFT) {
+	size = gw_applet->container->allocation.width;
+
+	if (size > picwidth + textwidth + 18)
+		gw_applet->box = gtk_hbox_new (FALSE, 2);
+	else
+		gw_applet->box = gtk_vbox_new (FALSE, 2);
+    
+    } else {
+	gw_applet->box = gtk_vbox_new (FALSE, 2);
     }
 
-    gtk_widget_show_all (GTK_WIDGET (gw_applet->applet));
+    /* Rebuild the applet it's visual area */
+    gtk_container_add (GTK_CONTAINER (gw_applet->container), gw_applet->box);
 
+    gtk_box_pack_start (GTK_BOX (gw_applet->box), gw_applet->image, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (gw_applet->box), gw_applet->label, FALSE, FALSE, 0);
+
+    /* Update temperature text */
+    temp = weather_info_get_temp_summary(gw_applet->gweather_info);
+    if (temp) 
+    	gtk_label_set_text(GTK_LABEL(gw_applet->label), temp);
+
+    gtk_widget_show_all (GTK_WIDGET (gw_applet->applet));
 }
 
 static void change_orient_cb (PanelApplet *w, PanelAppletOrient o, gpointer data)
@@ -90,6 +113,36 @@ static void change_size_cb(PanelApplet *w, gint s, gpointer data)
     return;
 }
 
+static void change_background_cb (
+				PanelApplet *a, 
+				PanelAppletBackgroundType type,
+				GdkColor *color, GdkPixmap *pixmap, 
+				gpointer data)
+{
+	GWeatherApplet *gw_applet = (GWeatherApplet *) data;
+	GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+	switch (type) {
+		case PANEL_PIXMAP_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
+			break;
+
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (gw_applet->applet), GTK_STATE_NORMAL, color);
+			break;
+
+		case PANEL_NO_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
+			break;
+
+		default:
+			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
+			break;
+	}
+
+	gtk_rc_style_unref (rc_style);
+}
+
 
 static gboolean clicked_cb (GtkWidget *widget, GdkEventButton *ev, gpointer data)
 {
@@ -152,7 +205,7 @@ static void help_cb (BonoboUIComponent *uic,
     GError *error = NULL;
 
     egg_help_display_on_screen (
-		"gweather", "gweather-prefs",
+		"gweather", NULL,
 		gtk_widget_get_screen (GTK_WIDGET (gw_applet->applet)),
 		&error);
 
@@ -206,6 +259,36 @@ applet_destroy (GtkWidget *widget, GWeatherApplet *gw_applet)
 
     if (gw_applet->gweather_dialog)
        gtk_widget_destroy (gw_applet->gweather_dialog);
+
+    if (gw_applet->timeout_tag > 0) {
+       gtk_timeout_remove(gw_applet->timeout_tag);
+       gw_applet->timeout_tag = 0;
+    }
+	
+    if (gw_applet->gweather_info->metar_handle) {
+       gnome_vfs_async_cancel(gw_applet->gweather_info->metar_handle);
+       gw_applet->gweather_info->metar_handle = 0;
+    }
+
+    if (gw_applet->gweather_info->iwin_handle) {
+       gnome_vfs_async_cancel(gw_applet->gweather_info->iwin_handle);
+       gw_applet->gweather_info->iwin_handle = 0;
+    }
+
+    if (gw_applet->gweather_info->wx_handle) {
+       gnome_vfs_async_cancel(gw_applet->gweather_info->wx_handle);
+       gw_applet->gweather_info->wx_handle = 0;
+    }
+
+    if (gw_applet->gweather_info->met_handle) {
+       gnome_vfs_async_cancel(gw_applet->gweather_info->met_handle);
+       gw_applet->gweather_info->met_handle = 0;
+    }
+
+    if (gw_applet->gweather_info->bom_handle) {
+       gnome_vfs_async_cancel(gw_applet->gweather_info->bom_handle);
+       gw_applet->gweather_info->bom_handle = 0;
+    }
 }
 
 void gweather_applet_create (GWeatherApplet *gw_applet)
@@ -231,11 +314,16 @@ void gweather_applet_create (GWeatherApplet *gw_applet)
         gnome_window_icon_set_default_from_file (gtk_icon_info_get_filename (icon_info));
         gtk_icon_info_free (icon_info);
     }
-    
+
+    gw_applet->container = gtk_hbox_new (FALSE, 0);
+    gtk_container_add (GTK_CONTAINER (gw_applet->applet), gw_applet->container);
+
     g_signal_connect (G_OBJECT(gw_applet->applet), "change_orient",
                        G_CALLBACK(change_orient_cb), gw_applet);
     g_signal_connect (G_OBJECT(gw_applet->applet), "change_size",
                        G_CALLBACK(change_size_cb), gw_applet);
+    g_signal_connect (G_OBJECT(gw_applet->applet), "change_background",
+		       G_CALLBACK(change_background_cb), gw_applet);
     g_signal_connect (G_OBJECT(gw_applet->applet), "destroy", 
                        G_CALLBACK (applet_destroy), gw_applet);
     gtk_signal_connect (GTK_OBJECT(gw_applet->applet), "button_press_event",
