@@ -29,6 +29,7 @@
 
 #include "popcheck.h"
 #include "remote-helper.h"
+#include "mailcheck.h"
 
 #include "egg-screen-help.h"
 #include "egg-screen-exec.h"
@@ -133,6 +134,9 @@ struct _MailCheck {
 	/* for the selection routine */
 	char *selected_pixmap_name;
 
+	/* the about box */
+	GtkWidget *about;
+
 	/* The property window */
 	GtkWidget *property_window;
 	GtkWidget *min_spin, *sec_spin;
@@ -140,9 +144,6 @@ struct _MailCheck {
 	GtkWidget *newmail_cmd_entry, *newmail_cmd_check;
 	GtkWidget *clicked_cmd_entry, *clicked_cmd_check;
 
-	/* the about box */
-	GtkWidget *about;
-
 	GtkWidget *password_dialog;
 
 	gboolean anim_changed;
@@ -650,8 +651,9 @@ mailcheck_execute_shell (MailCheck  *mailcheck,
 {
 	GError *error = NULL;
 
-	egg_screen_execute_command_line_async (
-		gtk_widget_get_screen (GTK_WIDGET (mailcheck->applet)), command, &error);
+	gdk_spawn_command_line_on_screen (gtk_widget_get_screen (GTK_WIDGET (mailcheck->applet)),
+					  command, &error); 
+
 	if (error) {
 		GtkWidget *dialog;
 		char *msg;
@@ -681,6 +683,14 @@ mailcheck_execute_shell (MailCheck  *mailcheck,
 	}
 }
 
+void
+command_execute_shell (gpointer data,
+                       const char *command)
+{
+        MailCheck *mc = (MailCheck *) data;
+        mailcheck_execute_shell (mc, command);
+}
+
 static G_CONST_RETURN char *
 mail_animation_filename (MailCheck *mc)
 {
@@ -1080,8 +1090,8 @@ after_mail_check (MailCheck *mc)
 		}
 		else {
 			if (mc->unreadmail) {
-				plural1 = g_strdup(ngettext ("%d unread", "%d unread",  mc->unreadmail));
-				plural2 = g_strdup(ngettext ("%d message", "%d messages", mc->totalmail));
+				plural1 = g_strdup_printf(ngettext ("%d unread", "%d unread",  mc->unreadmail), mc->unreadmail);
+				plural2 = g_strdup_printf(ngettext ("%d message", "%d messages", mc->totalmail), mc->totalmail);
 
 				/* translators: this is of the form "%d unread/%d messages" */
 				text = g_strdup_printf(_("%s/%s"), plural1, plural2);
@@ -1179,7 +1189,8 @@ mail_check_timeout (gpointer data)
 
 		mailcheck_execute_shell (mc, mc->pre_check_cmd);
 
-		mc->mail_timeout = gtk_timeout_add(mc->update_freq, mail_check_timeout, mc);
+		if(mc->auto_update)
+			mc->mail_timeout = gtk_timeout_add(mc->update_freq, mail_check_timeout, mc);
 	}
 
 	check_mail_file_status (mc);
@@ -2452,6 +2463,7 @@ mailcheck_about(BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 	if (mc->about) {
 		gtk_window_set_screen (GTK_WINDOW (mc->about),
 				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
+
 		gtk_window_present (GTK_WINDOW (mc->about));
 		return;
 	}
@@ -2469,6 +2481,7 @@ mailcheck_about(BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 				     pixbuf);
 				     
 	gtk_window_set_wmclass (GTK_WINDOW (mc->about), "mailcheck", "Mailcheck");
+
 	gtk_window_set_screen (GTK_WINDOW (mc->about),
 			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 
@@ -2477,6 +2490,7 @@ mailcheck_about(BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 
 	g_signal_connect( G_OBJECT(mc->about), "destroy",
 			    G_CALLBACK(gtk_widget_destroyed), &mc->about );
+
 	gtk_widget_show(mc->about);
 }
 
@@ -2530,6 +2544,39 @@ applet_change_orient(PanelApplet * w, PanelAppletOrient orient, gpointer data)
 
 }
 
+static void
+applet_change_background(PanelApplet *a,
+			 PanelAppletBackgroundType type,
+			 GdkColor *color, GdkPixmap *pixmap,
+			 MailCheck *mc)
+{
+	GtkRcStyle *rc_style = gtk_rc_style_new ();
+
+	switch (type) {
+		case PANEL_PIXMAP_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (mc->ebox), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (mc->applet), rc_style);
+			break;
+
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (mc->ebox), GTK_STATE_NORMAL, color);
+			gtk_widget_modify_bg (GTK_WIDGET (mc->applet), GTK_STATE_NORMAL, color);
+			break;
+
+		case PANEL_NO_BACKGROUND:
+			gtk_widget_modify_style (GTK_WIDGET (mc->ebox), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (mc->applet), rc_style);
+			break;
+
+		default:
+			gtk_widget_modify_style (GTK_WIDGET (mc->ebox), rc_style);
+			gtk_widget_modify_style (GTK_WIDGET (mc->applet), rc_style);
+			break;
+	}
+
+	gtk_rc_style_unref (rc_style);
+}
+
 static void
 help_callback (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 {
@@ -2673,6 +2720,7 @@ create_mail_widgets (MailCheck *mc)
 
 	/* The label */
 	mc->label = gtk_label_new (NULL);
+	gtk_label_set_justify (GTK_LABEL (mc->label), GTK_JUSTIFY_CENTER);
 	gtk_label_set_line_wrap (GTK_LABEL (mc->label), TRUE);
 	gtk_widget_show (mc->label);
 	gtk_widget_ref (mc->label);
@@ -2800,15 +2848,22 @@ mailcheck_applet_fill (PanelApplet *applet)
 			 G_CALLBACK(applet_change_orient),
 			 mc);
 
+	g_signal_connect(G_OBJECT(applet), "change_background",
+			 G_CALLBACK(applet_change_background),
+			 mc);
+
 	mailcheck = create_mail_widgets (mc);
 	gtk_widget_show(mailcheck);
 
 	gtk_container_add (GTK_CONTAINER (applet), mailcheck);
 
 	g_signal_connect(G_OBJECT(mc->ebox), "button_press_event",
-			 G_CALLBACK(exec_clicked_cmd), mc);
+			 G_CALLBACK(exec_clicked_cmd),
+			 mc);
+
 	g_signal_connect(G_OBJECT(applet), "key_press_event",
-			 G_CALLBACK(key_press_cb), mc);
+			 G_CALLBACK(key_press_cb),
+			 mc);
 
 	panel_applet_setup_menu_from_file (applet,
 					   NULL,
